namespace :admin do
  desc "Assign admin role to a user by email"
  task :make_admin, [ :email ] => :environment do |t, args|
    email = args[:email]

    if email.blank?
      puts "Error: Email is required."
      puts "Usage: rake admin:make_admin[user@example.com]"
      exit 1
    end

    user = User.find_by(email: email)

    if user.nil?
      puts "Error: User with email '#{email}' not found."
      exit 1
    end

    # Get the admin role
    admin_role = Role.find_by(name: "admin")

    if admin_role.nil?
      puts "Error: 'admin' role does not exist."
      exit 1
    end

    if user.has_role?("admin")
      puts "User '#{user.name}' already has the admin role."
    else
      user.roles << admin_role
      puts "Successfully assigned admin role to '#{user.name}' (#{user.email})."
    end
  end

  desc "Assign shift coordinator role to a user by email"
  task :make_shift_coordinator, [ :email ] => :environment do |t, args|
    email = args[:email]

    if email.blank?
      puts "Error: Email is required."
      puts "Usage: rake admin:make_shift_coordinator[user@example.com]"
      exit 1
    end

    user = User.find_by(email: email)

    if user.nil?
      puts "Error: User with email '#{email}' not found."
      exit 1
    end

    # Get the shift_coordinator role
    shift_coordinator_role = Role.find_by(name: "shift_coordinator")

    if shift_coordinator_role.nil?
      puts "Error: 'shift_coordinator' role does not exist."
      exit 1
    end

    if user.has_role?("shift_coordinator")
      puts "User '#{user.name}' already has the shift coordinator role."
    else
      user.roles << shift_coordinator_role
      puts "Successfully assigned shift coordinator role to '#{user.name}' (#{user.email})."
    end
  end

  desc "List all users with special roles"
  task list: :environment do
    admin_role = Role.find_by(name: "admin")
    shift_coordinator_role = Role.find_by(name: "shift_coordinator")
    events_admin_role = Role.find_by(name: "events_admin")

    puts "Users with special roles:"
    puts "-----------------------"

    if admin_role && admin_role.users.any?
      puts "\nAdmins:"
      admin_role.users.each do |admin|
        puts "  #{admin.name} (#{admin.email || 'No email'})"
      end
    end

    if shift_coordinator_role && shift_coordinator_role.users.any?
      puts "\nShift Coordinators:"
      shift_coordinator_role.users.each do |sc|
        puts "  #{sc.name} (#{sc.email || 'No email'})"
      end
    end

    if events_admin_role && events_admin_role.users.any?
      puts "\nEvents Admins:"
      events_admin_role.users.each do |ea|
        puts "  #{ea.name} (#{ea.email || 'No email'})"
      end
    end

    if (admin_role.nil? || admin_role.users.empty?) &&
       (shift_coordinator_role.nil? || shift_coordinator_role.users.empty?) &&
       (events_admin_role.nil? || events_admin_role.users.empty?)
      puts "No users with special roles found."
    end
  end

  desc "Remove admin role from a user by email"
  task :remove_admin, [ :email ] => :environment do |t, args|
    email = args[:email]

    if email.blank?
      puts "Error: Email is required."
      puts "Usage: rake admin:remove_admin[user@example.com]"
      exit 1
    end

    user = User.find_by(email: email)

    if user.nil?
      puts "Error: User with email '#{email}' not found."
      exit 1
    end

    # Get the admin role
    admin_role = Role.find_by(name: "admin")

    if admin_role.nil?
      puts "Error: 'admin' role does not exist."
      exit 1
    end

    if !user.has_role?("admin")
      puts "User '#{user.name}' does not have the admin role."
    else
      user.roles.delete(admin_role)
      puts "Successfully removed admin role from '#{user.name}' (#{user.email})."
    end
  end

  desc "Remove shift coordinator role from a user by email"
  task :remove_shift_coordinator, [ :email ] => :environment do |t, args|
    email = args[:email]

    if email.blank?
      puts "Error: Email is required."
      puts "Usage: rake admin:remove_shift_coordinator[user@example.com]"
      exit 1
    end

    user = User.find_by(email: email)

    if user.nil?
      puts "Error: User with email '#{email}' not found."
      exit 1
    end

    # Get the shift_coordinator role
    shift_coordinator_role = Role.find_by(name: "shift_coordinator")

    if shift_coordinator_role.nil?
      puts "Error: 'shift_coordinator' role does not exist."
      exit 1
    end

    if !user.has_role?("shift_coordinator")
      puts "User '#{user.name}' does not have the shift coordinator role."
    else
      user.roles.delete(shift_coordinator_role)
      puts "Successfully removed shift coordinator role from '#{user.name}' (#{user.email})."
    end
  end
end
